@@ -6,11 +6,11 @@ from six.moves import xrange
 import copy
 
 def large_plot(n):
-    from bokeh.objects import (Plot, PlotContext, LinearAxis, Grid, Glyph,
+    from bokeh.models import (Plot, PlotContext, LinearAxis, Grid, GlyphRenderer,
         ColumnDataSource, DataRange1d, PanTool, WheelZoomTool, BoxZoomTool,
         BoxSelectTool, BoxSelectionOverlay, ResizeTool, PreviewSaveTool,
         ResetTool)
-    from bokeh.glyphs import Line
+    from bokeh.models.glyphs import Line
 
     context = PlotContext()
     objects = set([context])
@@ -25,7 +25,8 @@ def large_plot(n):
         xgrid = Grid(plot=plot, dimension=0)
         ygrid = Grid(plot=plot, dimension=1)
         tickers = [xaxis.ticker, xaxis.formatter, yaxis.ticker, yaxis.formatter]
-        renderer = Glyph(data_source=source, glyph=Line(x='x', y='y'))
+        glyph = Line(x='x', y='y')
+        renderer = GlyphRenderer(data_source=source, glyph=glyph)
         plot.renderers.append(renderer)
         pan = PanTool(plot=plot)
         wheel_zoom = WheelZoomTool(plot=plot)
@@ -38,7 +39,7 @@ def large_plot(n):
         tools = [pan, wheel_zoom, box_zoom, box_select, box_selection, resize, previewsave, reset]
         plot.tools.append(tools)
         context.children.append(plot)
-        objects |= set([source, xdr, ydr, plot, xaxis, yaxis, xgrid, ygrid, renderer] + tickers + tools)
+        objects |= set([source, xdr, ydr, plot, xaxis, yaxis, xgrid, ygrid, renderer, glyph, plot.tool_events] + tickers + tools)
 
     return context, objects
 
@@ -71,21 +72,6 @@ class TestViewable(unittest.TestCase):
 
 class TestCollectPlotObjects(unittest.TestCase):
 
-    def test_references(self):
-        from bokeh.plot_object import PlotObject
-        pobject1 = PlotObject()
-        pobject2 = PlotObject()
-        pobject3 = PlotObject()
-        pobject4 = PlotObject()
-        pobject1.pobject2 = pobject2
-        pobject1.pobject3 = pobject3
-        pobject3.pobject4 = pobject4
-        pobject1.properties_with_refs = Mock(return_value=['pobject2', 'pobject3'])
-        pobject3.properties_with_refs = Mock(return_value=['pobject4'])
-        resultset = set(pobject1.references())
-        expectedset = set([pobject1, pobject2, pobject3, pobject4])
-        self.assertEqual(resultset, expectedset)
-
     def test_references_large(self):
         context, objects = large_plot(500)
         self.assertEqual(set(context.references()), objects)
@@ -93,7 +79,7 @@ class TestCollectPlotObjects(unittest.TestCase):
 class TestPlotObject(unittest.TestCase):
 
     def setUp(self):
-        from bokeh.objects import PlotObject
+        from bokeh.models import PlotObject
         self.pObjectClass = PlotObject
 
     def test_init(self):
@@ -128,7 +114,7 @@ class TestPlotObject(unittest.TestCase):
         self.assertEqual(obj.min_border, 100)
 
     def test_references_by_ref_by_value(self):
-        from bokeh.objects import PlotObject
+        from bokeh.models import PlotObject
         from bokeh.properties import HasProps, Instance, Int
 
         class T(PlotObject):
@@ -162,7 +148,7 @@ class TestPlotObject(unittest.TestCase):
         self.assertEqual(x2.references(), {t1, y, t2, z2, x2})
 
     def test_references_in_containers(self):
-        from bokeh.objects import PlotObject
+        from bokeh.models import PlotObject
         from bokeh.properties import Int, String, Instance, List, Tuple, Dict
 
         # XXX: can't use Y, because of:
