@@ -3,23 +3,24 @@ from __future__ import print_function
 from bokeh.browserlib import view
 from bokeh.document import Document
 from bokeh.embed import file_html
-from bokeh.glyphs import Circle
-from bokeh.objects import (
-    GMapPlot, Range1d, ColumnDataSource,
+from bokeh.models.glyphs import Circle
+from bokeh.models import (
+    GMapPlot, Range1d, ColumnDataSource, LinearAxis,
     PanTool, WheelZoomTool, BoxSelectTool,
-    BoxSelectionOverlay, MapOptions)
+    BoxSelectionOverlay, GMapOptions)
 from bokeh.resources import INLINE
 
 x_range = Range1d()
 y_range = Range1d()
 
-map_options = MapOptions(lat=30.2861, lng=-97.7394, zoom=15)
+map_options = GMapOptions(lat=30.2861, lng=-97.7394, zoom=15)
 
 plot = GMapPlot(
     x_range=x_range, y_range=y_range,
     map_options=map_options,
     title = "Austin"
 )
+plot.map_options.map_type="hybrid"
 
 source = ColumnDataSource(
     data=dict(
@@ -38,6 +39,12 @@ box_select = BoxSelectTool()
 
 plot.add_tools(pan, wheel_zoom, box_select)
 
+xaxis = LinearAxis(axis_label="lat", major_tick_in=0)
+plot.add_layout(xaxis, 'below')
+
+yaxis = LinearAxis(axis_label="lon", major_tick_in=0)
+plot.add_layout(yaxis, 'left')
+
 overlay = BoxSelectionOverlay(tool=box_select)
 plot.add_layout(overlay)
 
