@@ -7,4 +7,4 @@ class TestExamples(unittest.TestCase):
     def test_examples(self):
         script = os.path.join(os.path.dirname(__file__), "test")
         proc = subprocess.Popen(["python", script])
-        self.assertTrue(proc.wait() == 0, "examples do *NOT* work properly")
+        self.assertEqual(proc.wait(), 0, "examples do *NOT* work properly")
